@@ -8,9 +8,10 @@ from collections import defaultdict
 
 from sql import Null, Literal
 from sql.aggregate import Sum, Max
+from sql.functions import CharLength
 from sql.conditionals import Coalesce, Case
 
-from trytond.model import ModelView, ModelSQL, fields, Check
+from trytond.model import ModelView, ModelSQL, fields, Check, DeactivableMixin
 from trytond.wizard import Wizard, StateTransition, StateView, StateAction, \
     StateReport, Button
 from trytond.report import Report
@@ -22,7 +23,7 @@ from trytond.rpc import RPC
 from trytond.tools import reduce_ids, grouped_slice
 from trytond.config import config
 
-__all__ = ['Move', 'Reconciliation', 'Line', 'OpenJournalAsk',
+__all__ = ['Move', 'Reconciliation', 'Line', 'WriteOff', 'OpenJournalAsk',
     'OpenJournal', 'OpenAccount',
     'ReconcileLinesWriteOff', 'ReconcileLines',
     'UnreconcileLines',
@@ -100,14 +101,18 @@ class Move(ModelSQL, ModelView):
         cls._buttons.update({
                 'post': {
                     'invisible': Eval('state') == 'posted',
+                    'depends': ['state'],
                     },
                 })
+        cls.__rpc__.update({
+                'post': RPC(
+                    readonly=False, instantiate=0, fresh_session=True),
+                })
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
         cursor = Transaction().connection.cursor()
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         sql_table = cls.__table__()
         pool = Pool()
         Period = pool.get('account.period')
@@ -115,14 +120,6 @@ class Move(ModelSQL, ModelView):
         FiscalYear = pool.get('account.fiscalyear')
         fiscalyear = FiscalYear.__table__()
 
-        # Migration from 2.4:
-        #   - name renamed into number
-        #   - reference renamed into post_number
-        if table.column_exist('name'):
-            table.column_rename('name', 'number')
-        if table.column_exist('reference'):
-            table.column_rename('reference', 'post_number')
-
         created_company = not table.column_exist('company')
 
         super(Move, cls).__register__(module_name)
@@ -136,12 +133,17 @@ class Move(ModelSQL, ModelView):
                     where=period.id == sql_table.period)
             cursor.execute(*sql_table.update([sql_table.company], [value]))
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         table.index_action(['journal', 'period'], 'add')
 
         # Add index on create_date
         table.index_action('create_date', action='add')
 
+    @classmethod
+    def order_post_number(cls, tables):
+        table, _ = tables[None]
+        return [CharLength(table.post_number), table.post_number]
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -237,7 +239,7 @@ class Move(ModelSQL, ModelView):
         all_moves = []
         args = []
         for moves, values in zip(actions, actions):
-            keys = values.keys()
+            keys = list(values.keys())
             for key in cls._check_modify_exclude:
                 if key in keys:
                     keys.remove(key)
@@ -261,7 +263,8 @@ class Move(ModelSQL, ModelView):
                         or Transaction().context.get('journal'))
                 if journal_id:
                     journal = Journal(journal_id)
-                    vals['number'] = Sequence.get_id(journal.sequence.id)
+                    if journal.sequence:
+                        vals['number'] = Sequence.get_id(journal.sequence.id)
 
         moves = super(Move, cls).create(vlist)
         cls.validate_move(moves)
@@ -276,25 +279,15 @@ class Move(ModelSQL, ModelView):
 
     @classmethod
     def copy(cls, moves, default=None):
-        Line = Pool().get('account.move.line')
-
         if default is None:
             default = {}
-        default = default.copy()
-        default['number'] = None
-        default['post_number'] = None
-        default['state'] = cls.default_state()
-        default['post_date'] = None
-        default['lines'] = None
-
-        new_moves = []
-        for move in moves:
-            new_move, = super(Move, cls).copy([move], default=default)
-            Line.copy(move.lines, default={
-                    'move': new_move.id,
-                    })
-            new_moves.append(new_move)
-        return new_moves
+        else:
+            default = default.copy()
+        default.setdefault('number', None)
+        default.setdefault('post_number', None)
+        default.setdefault('state', cls.default_state())
+        default.setdefault('post_date', None)
+        return super().copy(moves, default=default)
 
     @classmethod
     def validate_move(cls, moves):
@@ -371,24 +364,23 @@ class Move(ModelSQL, ModelView):
                     'date': date,
                     'period': period_id,
                     })
+        default['lines.debit'] = lambda data: data['debit'] * -1
+        default['lines.credit'] = lambda data: data['credit'] * -1
+        default['lines.amount_second_currency'] = (
+            lambda data: data['amount_second_currency'] * -1
+            if data['amount_second_currency']
+            else data['amount_second_currency'])
+        default['lines.tax_lines.amount'] = lambda data: data['amount'] * -1
         return default
 
     def cancel(self, default=None):
         'Return a cancel move'
         if default is None:
             default = {}
+        else:
+            default = default.copy()
         default.update(self._cancel_default())
         cancel_move, = self.copy([self], default=default)
-        for line in cancel_move.lines:
-            line.debit *= -1
-            line.credit *= -1
-            if line.second_currency:
-                line.amount_second_currency *= -1
-            for tax_line in line.tax_lines:
-                tax_line.amount *= -1
-            line.tax_lines = line.tax_lines  # Force tax_lines changing
-        cancel_move.lines = cancel_move.lines  # Force lines changing
-        cancel_move.save()
         return cancel_move
 
     @classmethod
@@ -462,7 +454,7 @@ class Reconciliation(ModelSQL, ModelView):
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
         cursor = Transaction().connection.cursor()
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         sql_table = cls.__table__()
         pool = Pool()
         Move = pool.get('account.move')
@@ -477,7 +469,7 @@ class Reconciliation(ModelSQL, ModelView):
         # Migration from 3.8: new date field
         if (not date_exist
                 and TableHandler.table_exist(Line._table)
-                and TableHandler(Line).column_exist('move')):
+                and Line.__table_handler__().column_exist('move')):
             cursor.execute(*sql_table.update(
                     [sql_table.date],
                     line.join(move,
@@ -542,15 +534,9 @@ class Reconciliation(ModelSQL, ModelView):
                             'party2': party.rec_name,
                             })
             if not account.company.currency.is_zero(debit - credit):
-                language = Transaction().language
-                languages = Lang.search([('code', '=', language)])
-                if not languages:
-                    languages = Lang.search([('code', '=', 'en')])
-                language = languages[0]
-                debit = Lang.currency(
-                    language, debit, account.company.currency)
-                credit = Lang.currency(
-                    language, credit, account.company.currency)
+                lang = Lang.get()
+                debit = lang.currency(debit, account.company.currency)
+                credit = lang.currency(credit, account.company.currency)
                 cls.raise_user_error('reconciliation_unbalanced', {
                         'debit': debit,
                         'credit': credit,
@@ -575,8 +561,18 @@ class Line(ModelSQL, ModelView):
         depends=['currency_digits', 'debit', 'tax_lines', 'journal'] +
         _depends)
     account = fields.Many2One('account.account', 'Account', required=True,
-            domain=[('kind', '!=', 'view')],
-            select=True, states=_states, depends=_depends)
+        domain=[
+            ('kind', '!=', 'view'),
+            ['OR',
+                ('start_date', '=', None),
+                ('start_date', '<=', Eval('date', None)),
+                ],
+            ['OR',
+                ('end_date', '=', None),
+                ('end_date', '>=', Eval('date', None)),
+                ],
+            ],
+        select=True, states=_states, depends=_depends + ['date'])
     move = fields.Many2One('account.move', 'Move', select=True, required=True,
         ondelete='CASCADE',
         states={
@@ -595,7 +591,7 @@ class Line(ModelSQL, ModelView):
             searcher='search_move_field')
     date = fields.Function(fields.Date('Effective Date', required=True,
             states=_states, depends=_depends),
-            'get_move_field', setter='set_move_field',
+            'on_change_with_date', setter='set_move_field',
             searcher='search_move_field')
     origin = fields.Function(fields.Reference('Origin',
             selection='get_origin'),
@@ -640,7 +636,6 @@ class Line(ModelSQL, ModelView):
     party_required = fields.Function(fields.Boolean('Party Required'),
         'on_change_with_party_required')
     maturity_date = fields.Date('Maturity Date',
-        states=_states, depends=_depends,
         help='This field is used for payable and receivable lines. \n'
         'You can put the limit date for the payment.')
     state = fields.Selection([
@@ -673,7 +668,8 @@ class Line(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Line, cls).__setup__()
-        cls._check_modify_exclude = {'reconciliation'}
+        cls._check_modify_exclude = {
+            'maturity_date', 'reconciliation', 'tax_lines'}
         cls._reconciliation_modify_disallow = {
             'account', 'debit', 'credit', 'party',
             }
@@ -702,8 +698,6 @@ class Line(ModelSQL, ModelView):
                     'no journal defined.'),
                 'move_view_account': ('You can not create a move line with '
                     'account "%s" because it is a view account.'),
-                'move_inactive_account': ('You can not create a move line '
-                    'with account "%s" because it is inactive.'),
                 'already_reconciled': 'Line "%s" (%d) already reconciled.',
                 'party_required': 'Party is required on line "%s"',
                 'party_set': 'Party must not be set on line "%s"',
@@ -711,27 +705,12 @@ class Line(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table = TableHandler(cls, module_name)
-
-        # Migration from 2.4: reference renamed into description
-        if table.column_exist('reference'):
-            table.column_rename('reference', 'description')
-
         super(Line, cls).__register__(module_name)
 
-        table = TableHandler(cls, module_name)
+        table = cls.__table_handler__(module_name)
         # Index for General Ledger
         table.index_action(['move', 'account'], 'add')
 
-        # Migration from 1.2
-        table.not_null_action('blocked', action='remove')
-
-        # Migration from 2.4: remove name, active
-        table.not_null_action('name', action='remove')
-        table.not_null_action('active', action='remove')
-        table.index_action('active', action='remove')
-
     @classmethod
     def default_date(cls):
         '''
@@ -870,6 +849,11 @@ class Line(ModelSQL, ModelView):
             return value.id
         return value
 
+    @fields.depends('move', '_parent_move.date')
+    def on_change_with_date(self, name=None):
+        if self.move:
+            return self.move.date
+
     @classmethod
     def set_move_field(cls, lines, name, value):
         if name.startswith('move_'):
@@ -978,7 +962,7 @@ class Line(ModelSQL, ModelView):
                 fiscalyear_id = fiscalyears[0].id
             else:
                 fiscalyear_id = -1
-            fiscalyear_ids = map(int, fiscalyears)
+            fiscalyear_ids = list(map(int, fiscalyears))
             where &= period.fiscalyear == fiscalyear_id
             where &= move.date <= date
         elif fiscalyear_id or period_ids or from_date or to_date:
@@ -998,7 +982,7 @@ class Line(ModelSQL, ModelView):
                     ('state', '=', 'open'),
                     ('company', '=', company),
                     ])
-            fiscalyear_ids = map(int, fiscalyears)
+            fiscalyear_ids = list(map(int, fiscalyears))
 
         # Use LEFT JOIN to allow database optimization
         # if no joined table is used in the where clause.
@@ -1024,9 +1008,6 @@ class Line(ModelSQL, ModelView):
         if self.account.kind in ('view',):
             self.raise_user_error('move_view_account', (
                     self.account.rec_name,))
-        if not self.account.active:
-            self.raise_user_error('move_inactive_account', (
-                    self.account.rec_name,))
         if bool(self.party) != bool(self.account.party_required):
             error = 'party_set' if self.party else 'party_required'
             self.raise_user_error(error, self.rec_name)
@@ -1148,10 +1129,10 @@ class Line(ModelSQL, ModelView):
     def copy(cls, lines, default=None):
         if default is None:
             default = {}
-        if 'move' not in default:
-            default['move'] = None
-        if 'reconciliation' not in default:
-            default['reconciliation'] = None
+        else:
+            default = default.copy()
+        default.setdefault('move', None)
+        default.setdefault('reconciliation', None)
         return super(Line, cls).copy(lines, default=default)
 
     @classmethod
@@ -1183,78 +1164,121 @@ class Line(ModelSQL, ModelView):
         return toolbar
 
     @classmethod
-    def reconcile(cls, lines, journal=None, date=None, account=None,
-            description=None):
+    def reconcile(
+            cls, *lines_list, date=None, writeoff=None, description=None):
+        """
+        Reconcile each list of lines together.
+        The writeoff keys are: date, method and description.
+        """
         pool = Pool()
-        Move = pool.get('account.move')
         Reconciliation = pool.get('account.move.reconciliation')
-        Period = pool.get('account.period')
-        Date = pool.get('ir.date')
 
-        for line in lines:
-            if line.reconciliation:
-                cls.raise_user_error('already_reconciled',
-                        error_args=(line.move.number, line.id,))
+        reconciliations = []
+        for lines in lines_list:
+            for line in lines:
+                if line.reconciliation:
+                    cls.raise_user_error('already_reconciled',
+                            error_args=(line.move.number, line.id,))
 
-        lines = list(lines)
-        reconcile_account = None
-        reconcile_party = None
-        amount = Decimal('0.0')
-        for line in lines:
-            amount += line.debit - line.credit
-            if not reconcile_account:
-                reconcile_account = line.account
-            if not reconcile_party:
-                reconcile_party = line.party
-        amount = reconcile_account.currency.round(amount)
-        if not account and journal:
-            if amount >= 0:
-                account = journal.debit_account
-            else:
-                account = journal.credit_account
-        if journal and account:
-            if not date:
-                date = Date.today()
-            period_id = Period.find(reconcile_account.company.id, date=date)
-            move, = Move.create([{
-                        'journal': journal.id,
-                        'period': period_id,
-                        'date': date,
-                        'description': description,
-                        'lines': [
-                            ('create', [{
-                                        'account': reconcile_account.id,
-                                        'party': (reconcile_party.id
-                                            if reconcile_party else None),
-                                        'debit': (amount < Decimal('0.0')
-                                            and - amount or Decimal('0.0')),
-                                        'credit': (amount > Decimal('0.0')
-                                            and amount or Decimal('0.0')),
-                                        }, {
-                                        'account': account.id,
-                                        'party': (reconcile_party.id
-                                            if (account.party_required and
-                                                reconcile_party)
-                                            else None),
-                                        'debit': (amount > Decimal('0.0')
-                                            and amount or Decimal('0.0')),
-                                        'credit': (amount < Decimal('0.0')
-                                            and - amount or Decimal('0.0')),
-                                        }]),
-                            ],
-                        }])
-            lines += cls.search([
-                    ('move', '=', move.id),
-                    ('account', '=', reconcile_account.id),
-                    ('debit', '=', amount < Decimal('0.0') and - amount
-                        or Decimal('0.0')),
-                    ('credit', '=', amount > Decimal('0.0') and amount
-                        or Decimal('0.0')),
-                    ], limit=1)
-        return Reconciliation.create([{
+            lines = list(lines)
+            reconcile_account = None
+            reconcile_party = None
+            amount = Decimal('0.0')
+            for line in lines:
+                amount += line.debit - line.credit
+                if not reconcile_account:
+                    reconcile_account = line.account
+                if not reconcile_party:
+                    reconcile_party = line.party
+            if amount:
+                move = cls._get_writeoff_move(
+                    reconcile_account, reconcile_party, amount,
+                    date, writeoff, description)
+                move.save()
+                lines += cls.search([
+                        ('move', '=', move.id),
+                        ('account', '=', reconcile_account.id),
+                        ('debit', '=', amount < Decimal('0.0') and - amount
+                            or Decimal('0.0')),
+                        ('credit', '=', amount > Decimal('0.0') and amount
+                            or Decimal('0.0')),
+                        ], limit=1)
+            reconciliations.append({
                     'lines': [('add', [x.id for x in lines])],
                     'date': max(l.date for l in lines),
-                    }])[0]
+                    })
+        return Reconciliation.create(reconciliations)
+
+    @classmethod
+    def _get_writeoff_move(cls, reconcile_account, reconcile_party, amount,
+            date=None, writeoff=None, description=None):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        Period = pool.get('account.period')
+        Move = pool.get('account.move')
+        if not date:
+            date = Date.today()
+        period_id = Period.find(reconcile_account.company.id, date=date)
+        account = None
+        journal = None
+        if writeoff:
+            if amount >= 0:
+                account = writeoff.debit_account
+            else:
+                account = writeoff.credit_account
+            journal = writeoff.journal
+
+        move = Move()
+        move.journal = journal
+        move.period = period_id
+        move.date = date
+        move.description = description
+
+        lines = []
+
+        line = cls()
+        lines.append(line)
+        line.account = reconcile_account
+        line.party = reconcile_party
+        line.debit = -amount if amount < 0 else 0
+        line.credit = amount if amount > 0 else 0
+
+        line = cls()
+        lines.append(line)
+        line.account = account
+        line.party = reconcile_party if account.party_required else None
+        line.debit = amount if amount > 0 else 0
+        line.credit = -amount if amount < 0 else 0
+
+        move.lines = lines
+        return move
+
+
+class WriteOff(DeactivableMixin, ModelSQL, ModelView):
+    'Reconcile Write Off'
+    __name__ = 'account.move.reconcile.write_off'
+    company = fields.Many2One('company.company', "Company", required=True)
+    name = fields.Char("Name", required=True, translate=True)
+    journal = fields.Many2One('account.journal', "Journal", required=True,
+        domain=[('type', '=', 'write-off')])
+    credit_account = fields.Many2One('account.account', "Credit Account",
+        required=True,
+        domain=[
+            ('kind', '!=', 'view'),
+            ('company', '=', Eval('company')),
+            ],
+        depends=['company'])
+    debit_account = fields.Many2One('account.account', "Debit Account",
+        required=True,
+        domain=[
+            ('kind', '!=', 'view'),
+            ('company', '=', Eval('company')),
+            ],
+        depends=['company'])
+
+    @classmethod
+    def default_company(cls):
+        return Transaction().context.get('company')
 
 
 class OpenJournalAsk(ModelView):
@@ -1382,14 +1406,17 @@ class OpenAccount(Wizard):
 class ReconcileLinesWriteOff(ModelView):
     'Reconcile Lines Write-Off'
     __name__ = 'account.move.reconcile_lines.writeoff'
-    journal = fields.Many2One('account.journal', 'Journal', required=True,
+    company = fields.Many2One('company.company', "Company", readonly=True)
+    writeoff = fields.Many2One('account.move.reconcile.write_off', "Write Off",
         domain=[
-            ('type', '=', 'write-off'),
-            ])
+            ('company', '=', Eval('company')),
+            ],
+        depends=['company'])
     date = fields.Date('Date', required=True)
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
         readonly=True, depends=['currency_digits'])
-    currency_digits = fields.Integer('Currency Digits', readonly=True)
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
     description = fields.Char('Description')
 
     @staticmethod
@@ -1397,6 +1424,11 @@ class ReconcileLinesWriteOff(ModelView):
         Date = Pool().get('ir.date')
         return Date.today()
 
+    @fields.depends('company')
+    def on_change_with_currency_digits(self, name=None):
+        if self.company:
+            return self.company.currency.digits
+
 
 class ReconcileLines(Wizard):
     'Reconcile Lines'
@@ -1433,17 +1465,17 @@ class ReconcileLines(Wizard):
         amount, company = self.get_writeoff()
         return {
             'amount': amount,
-            'currency_digits': company.currency.digits,
+            'company': company.id,
             }
 
     def transition_reconcile(self):
         Line = Pool().get('account.move.line')
 
-        journal = getattr(self.writeoff, 'journal', None)
+        writeoff = getattr(self.writeoff, 'writeoff', None)
         date = getattr(self.writeoff, 'date', None)
         description = getattr(self.writeoff, 'description', None)
         Line.reconcile(Line.browse(Transaction().context['active_ids']),
-            journal=journal, date=date, description=description)
+            writeoff=writeoff, date=date, description=description)
         return 'end'
 
 
@@ -1473,7 +1505,7 @@ class Reconcile(Wizard):
     show = StateView('account.reconcile.show',
         'account.reconcile_show_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Skip', 'next_', 'tryton-go-next'),
+            Button('Skip', 'next_', 'tryton-forward'),
             Button('Reconcile', 'reconcile', 'tryton-ok', default=True),
             ])
     reconcile = StateTransition()
@@ -1489,17 +1521,27 @@ class Reconcile(Wizard):
         cursor = Transaction().connection.cursor()
         account_rule = Rule.query_get(Account.__name__)
 
+        context = Transaction().context
+        if context['active_model'] == Line.__name__:
+            lines = [l for l in Line.browse(context['active_ids'])
+                if not l.reconciliation]
+            return list({l.account for l in lines if l.account.reconcile})
+
         balance = line.debit - line.credit
         cursor.execute(*line.join(account,
                 condition=line.account == account.id).select(
                 account.id,
                 where=((line.reconciliation == Null) & account.reconcile
                     & account.id.in_(account_rule)),
-                group_by=account.id,
-                having=(
-                    Sum(Case((balance > 0, 1), else_=0)) > 0)
-                & (Sum(Case((balance < 0, 1), else_=0)) > 0)
-                ))
+                group_by=[account.id, account.kind],
+                having=((
+                        Sum(Case((balance > 0, 1), else_=0)) > 0)
+                    & (Sum(Case((balance < 0, 1), else_=0)) > 0)
+                    | (Case((account.kind == 'receivable', Sum(balance) < 0),
+                            else_=False))
+                    | (Case((account.kind == 'payable', Sum(balance) > 0),
+                            else_=False))
+                    )))
         return [a for a, in cursor.fetchall()]
 
     def get_parties(self, account):
@@ -1509,15 +1551,25 @@ class Reconcile(Wizard):
         line = Line.__table__()
         cursor = Transaction().connection.cursor()
 
+        context = Transaction().context
+        if context['active_model'] == Line.__name__:
+            lines = [l for l in Line.browse(context['active_ids'])
+                if not l.reconciliation]
+            return list({l.party for l in lines if l.account == account})
+
         balance = line.debit - line.credit
         cursor.execute(*line.select(line.party,
                 where=(line.reconciliation == Null)
                 & (line.account == account.id),
                 group_by=line.party,
-                having=(
-                    Sum(Case((balance > 0, 1), else_=0)) > 0)
-                & (Sum(Case((balance < 0, 1), else_=0)) > 0)
-                ))
+                having=((
+                        Sum(Case((balance > 0, 1), else_=0)) > 0)
+                    & (Sum(Case((balance < 0, 1), else_=0)) > 0)
+                    | (Case((account.kind == 'receivable', Sum(balance) < 0),
+                            else_=False))
+                    | (Case((account.kind == 'payable', Sum(balance) > 0),
+                            else_=False))
+                    )))
         return [p for p, in cursor.fetchall()]
 
     def transition_next_(self):
@@ -1564,7 +1616,7 @@ class Reconcile(Wizard):
         defaults['party'] = self.show.party.id if self.show.party else None
         defaults['currency_digits'] = self.show.account.company.currency.digits
         defaults['lines'] = self._default_lines()
-        defaults['write_off'] = Decimal(0)
+        defaults['write_off_amount'] = Decimal(0)
         defaults['date'] = Date.today()
         return defaults
 
@@ -1581,6 +1633,16 @@ class Reconcile(Wizard):
 
     def _default_lines(self):
         'Return the larger list of lines which can be reconciled'
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        context = Transaction().context
+        if context['active_model'] == Line.__name__:
+            requested = {l for l in Line.browse(context['active_ids'])
+                if l.account == self.show.account
+                and l.party == self.show.party}
+        else:
+            requested = None
+
         currency = self.show.account.company.currency
         chunk = config.getint('account', 'reconciliation_chunk', default=10)
         # Combination is exponential so it must be limited to small number
@@ -1588,8 +1650,10 @@ class Reconcile(Wizard):
         for lines in grouped_slice(self._all_lines(), chunk):
             lines = list(lines)
             best = None
-            for n in xrange(len(lines), 1, -1):
+            for n in range(len(lines), 1, -1):
                 for comb_lines in combinations(lines, n):
+                    if requested and not requested.intersection(comb_lines):
+                        continue
                     amount = sum((l.debit - l.credit) for l in comb_lines)
                     if currency.is_zero(amount):
                         best = [l.id for l in comb_lines]
@@ -1598,6 +1662,8 @@ class Reconcile(Wizard):
                     break
             if best:
                 default.extend(best)
+        if not default and requested:
+            default = list(map(int, requested))
         return default
 
     def transition_reconcile(self):
@@ -1606,8 +1672,8 @@ class Reconcile(Wizard):
 
         if self.show.lines:
             Line.reconcile(self.show.lines,
-                journal=self.show.journal,
                 date=self.show.date,
+                writeoff=self.show.write_off,
                 description=self.show.description)
         return 'next_'
 
@@ -1630,31 +1696,34 @@ class ReconcileShow(ModelView):
         depends=['account', 'party'])
 
     _write_off_states = {
-        'required': Bool(Eval('write_off', 0)),
-        'invisible': ~Eval('write_off', 0),
+        'required': Bool(Eval('write_off_amount', 0)),
+        'invisible': ~Eval('write_off_amount', 0),
         }
-    _write_off_depends = ['write_off']
+    _write_off_depends = ['write_off_amount']
 
-    write_off = fields.Function(fields.Numeric('Write-Off',
+    write_off_amount = fields.Function(fields.Numeric('Amount',
             digits=(16, Eval('currency_digits', 2)),
             states=_write_off_states,
             depends=_write_off_depends + ['currency_digits']),
-        'on_change_with_write_off')
+        'on_change_with_write_off_amount')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
         'on_change_with_currency_digits')
-    journal = fields.Many2One('account.journal', 'Journal',
-        states=_write_off_states, depends=_write_off_depends,
-        domain=[
-            ('type', '=', 'write-off'),
-            ])
+    write_off = fields.Many2One(
+        'account.move.reconcile.write_off', "Write Off",
+        states=_write_off_states, depends=_write_off_depends)
     date = fields.Date('Date',
         states=_write_off_states, depends=_write_off_depends)
     description = fields.Char('Description',
-        states=_write_off_states, depends=_write_off_depends)
+        states={
+            'invisible': _write_off_states['invisible'],
+            }, depends=_write_off_depends)
 
-    @fields.depends('lines')
-    def on_change_with_write_off(self, name=None):
-        return sum((l.debit - l.credit) for l in self.lines)
+    @fields.depends('lines', 'currency_digits')
+    def on_change_with_write_off_amount(self, name=None):
+        digits = self.currency_digits or 0
+        exp = Decimal(str(10.0 ** -digits))
+        amount = sum(((l.debit - l.credit) for l in self.lines), Decimal(0))
+        return amount.quantize(exp)
 
     @fields.depends('account')
     def on_change_with_currency_digits(self, name=None):
@@ -1692,7 +1761,7 @@ class CancelMoves(Wizard):
             for line in move.lines + cancel_move.lines:
                 if line.account.reconcile:
                     to_reconcile[(line.account, line.party)].append(line)
-            for lines in to_reconcile.itervalues():
+            for lines in to_reconcile.values():
                 Line.reconcile(lines)
         return 'end'
 
